
from psycopg2.extras import RealDictCursor
from pg4nosql.PostgresNoSQLResultItem import PostgresNoSQLResultItem
from psycopg2.extensions import AsIs

__author__ = 'cansik'


class PostgresNoSQLQueryStructure(object):
    __SQL_QUERY_JSON = 'SELECT %s FROM %s WHERE %s'

    def __init__(self, name, connection):
        self.name = name
        self.connection = connection
        self.connection.cursor_factory = RealDictCursor
        self.cursor = self.connection.cursor()

    def query(self, query='True', columns='*'):
        self.cursor.execute(self.__SQL_QUERY_JSON,
                            (AsIs(columns), AsIs(self.name), AsIs(query)))
        rows = [item for item in self.cursor.fetchall()]
        items = map(lambda r: PostgresNoSQLResultItem(r, self), rows)
        return list(items)



import copy
import json
from psycopg2.extensions import AsIs
from psycopg2.extras import RealDictCursor
from pg4nosql import DEFAULT_JSON_COLUMN_NAME, DEFAULT_ROW_IDENTIFIER
from pg4nosql.PostgresNoSQLQueryStructure import PostgresNoSQLQueryStructure
from pg4nosql.PostgresNoSQLResultItem import PostgresNoSQLResultItem
from pg4nosql.PostgresNoSQLUtil import to_nullable_string


class PostgresNoSQLTable(PostgresNoSQLQueryStructure):
    __SQL_INSERT_JSON = "INSERT INTO %s(" + DEFAULT_JSON_COLUMN_NAME + " %s) VALUES(%s %s) RETURNING " + DEFAULT_ROW_IDENTIFIER
    __SQL_GET_JSON = 'SELECT * FROM %s WHERE ' + DEFAULT_ROW_IDENTIFIER + '=%s'
    __SQL_GET_COLUMNS = 'select column_name from information_schema.columns where table_name = %s'
    __SQL_DELETE_JSON = 'DELETE FROM %s WHERE ' + DEFAULT_ROW_IDENTIFIER + '=%s'
    __SQL_UPDATE_JSON = 'UPDATE %s SET ' + DEFAULT_JSON_COLUMN_NAME + '=%s %s WHERE ' + DEFAULT_ROW_IDENTIFIER + '=%s;'

    __SQL_INSERT = "INSERT INTO %s(%s) VALUES(%s) RETURNING " + DEFAULT_ROW_IDENTIFIER
    __SQL_UPDATE = 'UPDATE %s SET %s WHERE ' + DEFAULT_ROW_IDENTIFIER + '=%s;'

    __SQL_QUERY_WITH_JOIN = 'SELECT %s FROM %s AS a JOIN %s AS b ON %s WHERE %s'

    def __init__(self, name, connection):
        super(PostgresNoSQLTable, self).__init__(name, connection)
        self.super = super(PostgresNoSQLTable, self)

    def commit(self):
        """
        Use commit only if auto_commit in put or save are disabled!
        :return: None
        """
        self.connection.commit()

    def insert(self, auto_commit=True, **data):
        relational_data = data

        relational_data_columns = ''
        relational_data_values = ''

        if relational_data:
            relational_data_columns = ",".join(relational_data.keys())
            data_list = map(str, map(to_nullable_string, relational_data.values()))
            relational_data_values = ",".join(data_list)

        self.cursor.execute(self.__SQL_INSERT, (AsIs(self.name),
                                                AsIs(relational_data_columns),
                                                AsIs(relational_data_values)))

        if auto_commit:
            self.commit()

        return self.cursor.fetchone()[DEFAULT_ROW_IDENTIFIER]

    def update(self, object_id, auto_commit=True, **relational_data):
        relational_data_sql = ','.join(
            "%s=%s" % (key, str(to_nullable_string(val))) for (key, val) in relational_data.items())

        self.cursor.execute(self.__SQL_UPDATE, (AsIs(self.name),
                                                AsIs(relational_data_sql), object_id))

        if auto_commit:
            self.commit()

    def put(self, json_data, auto_commit=True, **relational_data):
        relational_data.update({DEFAULT_JSON_COLUMN_NAME: json_data})
        return self.insert(auto_commit=auto_commit, **relational_data)

    def save(self, record, auto_commit=True):
        data = copy.deepcopy(record.get_record())
        object_id = data.pop(DEFAULT_ROW_IDENTIFIER)
        self.update(object_id, auto_commit=auto_commit, **data)

    def get(self, object_id):
        self.cursor.execute(self.__SQL_GET_JSON, (AsIs(self.name), object_id))
        record = self.cursor.fetchone()

        if record is None:
            return record

        return PostgresNoSQLResultItem(record, self)

    def query_join(self, table_name, on_statement, query='True', columns='*'):
        self.cursor.execute(self.__SQL_QUERY_WITH_JOIN, (AsIs(columns),
                                                         AsIs(self.name),
                                                         AsIs(table_name),
                                                         AsIs(on_statement),
                                                         AsIs(query)))
        rows = [item for item in self.cursor.fetchall()]
        items = map(lambda r: PostgresNoSQLResultItem(r, self), rows)
        return items

    def query_one(self, query='True', columns='*'):
        result = self.query(query, columns)
        if not result:
            return None
        return result[0]

    def get_columns(self):
        self.cursor.execute(self.__SQL_GET_COLUMNS, (self.name,))
        columns = map(lambda m: m['column_name'], self.cursor.fetchall())
        return columns

    def delete(self, object_id, auto_commit=True):
        self.cursor.execute(self.__SQL_DELETE_JSON, (AsIs(self.name), object_id))
        if auto_commit:
            self.commit()

    def execute(self, sql_query):
        self.cursor.execute(sql_query)
        return self.cursor.fetchall()



from pg4nosql import DEFAULT_JSON_COLUMN_NAME, DEFAULT_ROW_IDENTIFIER


class PostgresNoSQLResultItem(object):

    def __init__(self, inner_result, origin_table):
        self.__inner_result = inner_result
        self.json = self.PostgresNoSQLJSONDocument(self.__inner_result)
        self.columns = self.__inner_result.keys()
        self.id = self.__inner_result.get(DEFAULT_ROW_IDENTIFIER, None)
        self.__origin_table = origin_table

    def __str__(self):
        return str(self.__inner_result)

    def __setitem__(self, key, value):
        self.__inner_result[key] = value

    def __getitem__(self, item):
        return self.__inner_result[item]

    def get_record(self):
        return self.__inner_result

    def save(self, auto_commit=True):
        if DEFAULT_ROW_IDENTIFIER not in self.columns:
            raise NotImplementedError('ResultItem does not contain column "%s"!' % DEFAULT_ROW_IDENTIFIER)
        self.__origin_table.save(self, auto_commit=auto_commit)

    class PostgresNoSQLJSONDocument(object):
        def __init__(self, inner_result):
            self.__inner_result = inner_result

        def __setitem__(self, key, value):
            self.__inner_result[DEFAULT_JSON_COLUMN_NAME][key] = value

        def __getitem__(self, item):
            return self.__inner_result[DEFAULT_JSON_COLUMN_NAME][item]

        def __str__(self):
            return str(self.__inner_result[DEFAULT_JSON_COLUMN_NAME])



DEFAULT_POSTGRES_HOST = 'localhost'
DEFAULT_POSTGRES_DATABASE = 'postgres'
DEFAULT_JSON_COLUMN_NAME = 'json'
DEFAULT_ROW_IDENTIFIER = 'id'
DEFAULT_ROW_IDENTIFIER_TYPE = 'SERIAL'
DEFAULT_POSTGRES_PORT = 5432



from distutils.core import setup

try:
    import pypandoc
    description = pypandoc.convert('README.md', 'rst')
except:
    description = ''

setup(
    name='pg4nosql',
    version='0.4.2',
    packages=['pg4nosql'],
    url='https://github.com/cansik/pg4nosql',
    license='MIT License',
    author='Florian Bruggisser',
    author_email='florian@nexpose.ch',
    description='A simple psycopg2 based wrapper for nosql like database interaction with python.',
    long_description=description,
    download_url='https://github.com/cansik/pg4nosql/tarball/0.4.0',
    keywords=['postgres', 'no', 'sql', 'wrapper', 'database', 'json'],
    install_requires=['psycopg2'],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ]
)



from pg4nosql.PostgresNoSQLQueryStructure import PostgresNoSQLQueryStructure

__author__ = 'cansik'


class PostgresNoSQLView(PostgresNoSQLQueryStructure):
    def __init__(self, name, connection):
        super(PostgresNoSQLView, self).__init__(name, connection)



import json
from pg4nosql.PostgresNoSQLResultItem import PostgresNoSQLResultItem
from psycopg2.extensions import adapt, AsIs

__author__ = 'cansik'


def to_sql_string(obj):
    if obj is None:
        return AsIs(obj)
    return str(obj)


def to_nullable_string(obj):
    if obj is None:
        return 'Null'
    if isinstance(obj, dict) or isinstance(obj, list):
        return adapt(json.dumps(obj))
    return adapt(str(obj))
